@@ -81,7 +81,8 @@ cycleOf g x = cycleOf' x [] where
     cycleOf' y ys = let y' = y .^ g in if y' == x then reverse (y:ys) else cycleOf' y' (y:ys)
 
 instance (Ord a, Show a) => Show (Permutation a) where
-    show g = show (toCycles g)
+    show g | g == 1 = "1"
+           | otherwise = show (toCycles g)
 
 parity g = let cs = toCycles g in (length (concat cs) - length cs) `mod` 2
 -- parity' g = length (filter (even . length) $ toCycles g) `mod` 2
