@@ -27,7 +27,7 @@ instance Show HBasis where
     show J = "j"
     show K = "k"
 
-instance (Num k) => Algebra k HBasis where
+instance (Eq k, Num k) => Algebra k HBasis where
     unit x = x *> return One
     mult = linear mult'
          where mult' (One,b) = return b
@@ -69,7 +69,7 @@ class Algebra k a => HasConjugation k a where
 
 -- |If an algebra has a conjugation operation, then it has multiplicative inverses,
 -- via 1/x = conj x / sqnorm x
-instance (Fractional k, Ord a, Show a, HasConjugation k a) => Fractional (Vect k a) where
+instance (Eq k, Fractional k, Ord a, Show a, HasConjugation k a) => Fractional (Vect k a) where
     recip 0 = error "recip 0"
     recip x = (1 / sqnorm x) *> conj x
     fromRational q = fromRational q *> 1
@@ -79,10 +79,10 @@ scalarPart :: (Num k) => Quaternion k -> k
 scalarPart = coeff One
 
 -- |The vector part of the quaternion w+xi+yj+zk is xi+yj+zk. Also called the pure part.
-vectorPart :: (Num k) => Quaternion k -> Quaternion k
+vectorPart :: (Eq k, Num k) => Quaternion k -> Quaternion k
 vectorPart q = q - scalarPart q *> 1
 
-instance Num k => HasConjugation k HBasis where
+instance (Eq k, Num k) => HasConjugation k HBasis where
     conj = (>>= conj') where
         conj' One = return One
         conj' imag = -1 *> return imag
@@ -126,7 +126,7 @@ reprSO3' q = \x -> q^-1 * x * q
 -- This shows that the multiplicative group of unit quaternions is isomorphic to Spin3, the double cover of SO3.
 --
 -- @reprSO3 q@ returns the 3*3 matrix representing this map.
-reprSO3 :: (Fractional k) => Quaternion k -> [[k]]
+reprSO3 :: (Eq k, Fractional k) => Quaternion k -> [[k]]
 reprSO3 q = reprSO3' q `asMatrix` [i,j,k]
 -- It's clear from the definition that repr3' q leaves scalars invariant
 
@@ -145,7 +145,7 @@ reprSO4' (l,r) = \x -> l^-1 * x * r
 -- is isomorphic to Spin4, the double cover of SO4.
 --
 -- @reprSO4 (l,r)@ returns the 4*4 matrix representing this map.
-reprSO4 :: (Fractional k) => (Quaternion k, Quaternion k) -> [[k]]
+reprSO4 :: (Eq k, Fractional k) => (Quaternion k, Quaternion k) -> [[k]]
 reprSO4 (l,r) = reprSO4' (l,r) `asMatrix` [1,i,j,k]
 -- could consider checking that l,r are unit length - except that this is hard to achieve working over Q
 
@@ -164,7 +164,7 @@ k' = return (Dual K)
 
 -- Coalgebra structure on the dual vector space to the quaternions
 -- The comult is the transpose of mult
-instance Num k => Coalgebra k (Dual HBasis) where
+instance (Eq k, Num k) => Coalgebra k (Dual HBasis) where
     counit = unwrap . linear counit'
         where counit' (Dual One) = return ()
               counit' _ = zero
@@ -204,4 +204,4 @@ instance Num k => Coalgebra k HBasis where
     counit (V ts) = sum [x | (One,x) <- ts]
     comult = linear cm
         where cm m = if m == One then return (m,m) else return (m,One) <+> return (One,m)
--}
\ No newline at end of file
+-}
