@@ -1,6 +1,11 @@
 -- Copyright (c) David Amos, 2009. All rights reserved.
 
-module Math.Combinatorics.GraphAuts where
+module Math.Combinatorics.GraphAuts (isVertexTransitive, isEdgeTransitive,
+                                     isArcTransitive, is2ArcTransitive, is3ArcTransitive, isnArcTransitive,
+                                     isDistanceTransitive,
+                                     graphAuts, incidenceAuts,
+                                     graphIsos, incidenceIsos,
+                                     isGraphIso, isIncidenceIso) where
 
 import Data.Either (lefts)
 import qualified Data.List as L
