@@ -5,6 +5,8 @@
 
 module Math.Combinatorics.IncidenceAlgebra where
 
+import Math.Core.Utils
+
 import Math.Combinatorics.Digraph
 import Math.Combinatorics.Poset
 
@@ -111,14 +113,14 @@ instance (Eq k, Num k, Ord a) => Algebra k (Interval a) where
 
 
 -- |The unit of the incidence algebra of a poset
-unitIA :: (Eq k, Num k, Ord t) => Poset t -> Vect k (Interval t)
+unitIA :: (Eq k, Num k, Ord a) => Poset a -> Vect k (Interval a)
 unitIA poset@(Poset (set,_)) = sumv [return (Iv poset (x,x)) | x <- set]
 
-basisIA :: Num k => Poset t -> [Vect k (Interval t)]
+basisIA :: Num k => Poset a -> [Vect k (Interval a)]
 basisIA poset = [return (Iv poset xy) | xy <- intervals poset]
 
 -- |The zeta function of a poset
-zetaIA :: (Eq k, Num k, Ord t) => Poset t -> Vect k (Interval t)
+zetaIA :: (Eq k, Num k, Ord a) => Poset a -> Vect k (Interval a)
 zetaIA poset = sumv $ basisIA poset
 
 -- Then for example, zeta^2 counts the number of points in each interval
@@ -132,7 +134,7 @@ muIA1 poset@(Poset (set,po)) = sum [mu (x,y) *> return (Iv poset (x,y)) | x <- s
 
 -- calculate the mobius function of a poset, with memoization
 -- |The Mobius function of a poset
-muIA :: (Eq k, Num k, Ord t) => Poset t -> Vect k (Interval t)
+muIA :: (Eq k, Num k, Ord a) => Poset a -> Vect k (Interval a)
 muIA poset@(Poset (set,po)) = sumv [mus M.! (x,y) *> return (Iv poset (x,y)) | x <- set, y <- set]
     where mu (x,y) | x == y    = 1
                    | po x y    = negate $ sum [mus M.! (x,z) | z <- set, po x z, po z y, z /= y]
@@ -152,7 +154,7 @@ invIA1 f | f == zerov = error "invIA 0"
 -- Stanley, Enumerative Combinatorics I, p144
 -- |The inverse of an element in the incidence algebra of a poset.
 -- This is only defined for elements which are non-zero on all intervals (x,x)
-invIA :: (Eq k, Fractional k, Ord t) => Vect k (Interval t) -> Maybe (Vect k (Interval t))
+invIA :: (Eq k, Fractional k, Ord a) => Vect k (Interval a) -> Maybe (Vect k (Interval a))
 invIA f | f == zerov = Nothing -- error "invIA 0"
         | any (==0) [f' (x,x) | x <- set] = Nothing -- error "invIA: not invertible"
         | otherwise = Just g
@@ -163,23 +165,24 @@ invIA f | f == zerov = Nothing -- error "invIA 0"
                    | otherwise = (-1 / f' (x,x)) * sum [f' (x,z) * (g's M.! (z,y)) | z <- interval poset (x,y), x /= z]
           g's = M.fromList [(xy, g' xy) | xy <- intervals poset]
 
-invIA' f = case invIA f of
-           Just g -> g
-           Nothing -> error "invIA': not invertible"
+instance (Eq k, Fractional k, Ord a, Show a) => HasInverses (Vect k (Interval a)) where
+    inverse f = case invIA f of
+                Just g -> g
+                Nothing -> error "IncidenceAlgebra.inverse: not invertible"
 
 -- Then for example we can count multichains or chains using the incidence algebra - see Stanley
 
 -- |A function (ie element of the incidence algebra) that counts the total number of chains in each interval
-numChainsIA :: (Ord a) => Poset a -> Vect Q (Interval a)
-numChainsIA poset = invIA' (2 *> unitIA poset <-> zetaIA poset)
+numChainsIA :: (Ord a, Show a) => Poset a -> Vect Q (Interval a)
+numChainsIA poset = (2 *> unitIA poset <-> zetaIA poset)^-1
 
 -- The eta function on intervals (x,y) is 1 if x -< y (y covers x), 0 otherwise
 etaIA poset = let DG vs es = hasseDigraph poset
               in sumv [return (Iv poset (x,y)) | (x,y) <- es]
 
 -- |A function (ie element of the incidence algebra) that counts the number of maximal chains in each interval
-numMaximalChainsIA :: (Ord a) => Poset a -> Vect Q (Interval a)
-numMaximalChainsIA poset = invIA' (unitIA poset <-> etaIA poset)
+numMaximalChainsIA :: (Ord a, Show a) => Poset a -> Vect Q (Interval a)
+numMaximalChainsIA poset = (unitIA poset <-> etaIA poset)^-1
 
 
 -- In order to quickCheck this, we would need
