@@ -171,13 +171,13 @@ fromIndeps1 es is = fromBases es bs
 -- |Given a matrix, represented as a list of rows, number the columns [1..],
 -- and construct the matroid whose independent sets correspond to those sets of columns which are linearly independent
 -- (or in case there are repetitions, those multisets of columns which are sets, and which are linearly independent).
-vectorMatroid :: (Fractional k) => [[k]] -> Matroid Int
+vectorMatroid :: (Eq k, Fractional k) => [[k]] -> Matroid Int
 vectorMatroid = vectorMatroid' . L.transpose
 
 -- |Given a list of vectors (or rows of a matrix), number the vectors (rows) [1..], and construct the matroid whose independent sets
 -- correspond to those sets of vectors (rows) which are linearly independent
 -- (or in case there are repetitions, those multisets which are sets, and which are linearly independent).
-vectorMatroid' :: (Fractional k) => [[k]] -> Matroid Int
+vectorMatroid' :: (Eq k, Fractional k) => [[k]] -> Matroid Int
 vectorMatroid' vs = fromBases (map fst vs') bs
     where vs' = zip [1..] vs
           bs = dfs [] [([],[],vs')]
@@ -247,6 +247,13 @@ incidenceGraphB m = G.G vs' es'
           es' = L.sort [ [Left e, Right b] | b <- bs, e <- b ]
 -- incidence graph for the matroid considered as an incidence structure between elements and bases
 
+incidenceGraphC m = G.G vs' es'
+    where es = elements m
+          cs = L.sort $ circuits m
+          vs' = map Left es ++ map Right cs
+          es' = L.sort [ [Left e, Right c] | c <- cs, e <- c ]
+-- incidence graph for the matroid considered as an incidence structure between elements and circuits
+
 incidenceGraphH m = G.G vs' es'
     where es = elements m
           hs = L.sort $ hyperplanes m
@@ -266,12 +273,7 @@ isMatroidIso m1 m2 = isIncidenceIso (incidenceGraphH m1) (incidenceGraphH m2)
 
 -- |Return the automorphisms of the matroid.
 matroidAuts :: (Ord a) => Matroid a -> [Permutation a]
-matroidAuts m
-    | G.isConnected hgraph = incidenceAuts hgraph
-    | G.isConnected bgraph = incidenceAuts bgraph
-    | otherwise = error "matroidAuts: incidence graph is not connected"
-    where hgraph = incidenceGraphH m
-          bgraph = incidenceGraphB m
+matroidAuts m = incidenceAuts $ incidenceGraphH m
 -- Note that the results aren't always what one intuitively expects from the geometric representation.
 -- This is because geometric representations suggest additional structure beyond matroid structure.
 -- For example, for the Vamos matroid v8,
@@ -613,7 +615,7 @@ fromHyperplanes es hs = fromCocircuits es $ map complement hs
 --
 -- A multiset of points in k^n is said to be affinely dependent if it contains two identical points,
 -- or three collinear points, or four coplanar points, or ... - and affinely independent otherwise.
-affineMatroid :: (Fractional k) => [[k]] -> Matroid Int
+affineMatroid :: (Eq k, Fractional k) => [[k]] -> Matroid Int
 affineMatroid vs = vectorMatroid' $ map (1:) vs
 
 -- |fromGeoRep returns a matroid from a geometric representation consisting of dependent flats of various ranks.
@@ -790,11 +792,11 @@ dsum m1 m2 = fromBases es bs
 -- REPRESENTABILITY
 
 -- |@matroidPG n fq@ returns the projective geometry PG(n,Fq), where fq is a list of the elements of Fq
-matroidPG :: (Fractional a) => Int -> [a] -> Matroid Int
+matroidPG :: (Eq a, Fractional a) => Int -> [a] -> Matroid Int
 matroidPG n fq = vectorMatroid' $ ptsPG n fq
 
 -- |@matroidAG n fq@ returns the affine geometry AG(n,Fq), where fq is a list of the elements of Fq
-matroidAG :: (Fractional a) => Int -> [a] -> Matroid Int
+matroidAG :: (Eq a, Fractional a) => Int -> [a] -> Matroid Int
 matroidAG n fq = vectorMatroid' $ ptsAG n fq
 
 
@@ -889,7 +891,7 @@ representations2 fq m = [ L.transpose mx | d <- substStars dhash' fq, let mx = i
 
 -- |Find representations of the matroid m over fq. Specifically, this function will find one representative
 -- of each projective equivalence class of representation.
-representations :: (Fractional fq, Ord a) => [fq] -> Matroid a -> [[[fq]]]
+representations :: (Eq fq, Fractional fq, Ord a) => [fq] -> Matroid a -> [[[fq]]]
 representations fq m = map L.transpose $ representations' (reverse $ zip b ir) (zip b' dhash')
     where fq' = tail fq -- fq \ {0}
           b = head $ bases m
@@ -905,7 +907,7 @@ representations fq m = map L.transpose $ representations' (reverse $ zip b ir) (
           representations' ls [] = [map snd $ reverse ls]
 
 -- |Is the matroid representable over Fq? For example, to find out whether a matroid m is binary, evaluate @isRepresentable f2 m@.
-isRepresentable :: (Fractional fq, Ord a) => [fq] -> Matroid a -> Bool
+isRepresentable :: (Eq fq, Fractional fq, Ord a) => [fq] -> Matroid a -> Bool
 isRepresentable fq m = (not . null) (representations fq m)
 
 -- |A binary matroid is a matroid which is representable over F2
@@ -1137,7 +1139,7 @@ whitney1st m = alternatingSign $ map length $ L.groupBy eqfst $ L.sort [(rankfun
 
 -- TODO
 
--- 1. Sort out the isomorphism / automorphism code in the case where the incidence graph isn't connected
+-- 1. Sort out the isomorphism code in the case where the incidence graph isn't connected
 
 -- 2. We could generate the geometric representation from a matroid (provided its rank <= 4)
 -- geoRep m = filter (isDependent m) (flats m)
