@@ -1,11 +1,12 @@
 -- Copyright (c) 2011, David Amos. All rights reserved.
 
-{-# LANGUAGE NoMonomorphismRestriction, TupleSections #-}
+{-# LANGUAGE NoMonomorphismRestriction #-}
 
 
 module Math.Combinatorics.Poset where
 
 import Math.Common.ListSet as LS -- set operations on strictly ascending lists
+import Math.Core.Utils -- for set/multiset operations on ordered lists
 import Math.Algebra.Field.Base
 import Math.Combinatorics.FiniteGeometry
 import Math.Algebra.LinearAlgebra
@@ -102,7 +103,7 @@ posetB :: Int -> Poset [Int]
 posetB n = Poset ( powerset [1..n], LS.isSubset )
 
 
--- LATTICE OF PARTITIONS OF [1..N] ORDERED BY REFINEMENT
+-- LATTICE OF SET PARTITIONS OF [1..N] ORDERED BY REFINEMENT
 
 partitions [] = [[]]
 partitions [x] = [[[x]]]
@@ -113,7 +114,7 @@ partitions (x:xs) = let ps = partitions xs in
 isRefinement a b = and [or [acell `isSubset` bcell | bcell <- b] | acell <- a]
 -- if we know that a and b are appropriately sorted, then this can probably be done more efficiently
 
--- |posetP n is the lattice of partitions of [1..n] ordered by refinement
+-- |posetP n is the lattice of set partitions of [1..n], ordered by refinement
 posetP :: Int -> Poset [[Int]]
 posetP n = Poset ( partitions [1..n], isRefinement )
 
@@ -130,17 +131,57 @@ isInterval _ = True
 
 intervalPartitions2 [] = [[]]
 intervalPartitions2 [x] = [[[x]]]
-intervalPartitions2 (x:xs) = let ips = intervalPartitions xs in
+intervalPartitions2 (x:xs) = let ips = intervalPartitions2 xs in
     map ([x]:) ips ++ [ (x:head):tail | (head:tail) <- ips]
 -- we're guaranteed that x+1 is at the head of the head
 
 
+-- LATTICE OF INTEGER PARTITIONS OF N ORDERED BY REFINEMENT
+
+integerPartitions1 n = ips (reverse [1..n]) n
+    where ips [] 0 = [[]]
+          ips [] _ = []
+          ips (x:xs) n | x > n     = ips xs n
+                       | otherwise = map (x:) (ips (x:xs) (n-x)) ++ ips xs n
+
+-- For example, integerPartitions 5 -> [ [5], [4,1], [3,2], [3,1,1], [2,2,1], [2,1,1,1], [1,1,1,1,1] ]
+integerPartitions n = dfs ([],n,n)
+    where dfs (xs, 0, _) = [reverse xs]
+          dfs (xs, r, i) = concatMap dfs [ (i':xs, r-i', i') | i' <- reverse [1..min r i] ]
+
+isIPRefinement ys xs = dfs xs ys
+    where dfs (x:xs) (y:ys) | x < y = False
+                            | x == y = dfs xs ys
+                            | otherwise = or [dfs xs' ys' | y' <- y:ys, let ys' = L.delete y' (y:ys),
+                                                                        let xs' = insertDesc (x-y') xs]
+          dfs [] [] = True
+          insertDesc = L.insertBy (flip compare)
+
+{-
+-- In theory it feels like this ought to be faster for large n, but in practice it's unclear
+isIPRefinement2 ys xs = isIPRefinement (ys \\ xs) (xs \\ ys)
+    where (\\) = diffDesc
+-}
+
+-- |posetIP n is the poset of integer partitions of n, ordered by refinement
+posetIP :: Int -> Poset [Int]
+posetIP n = Poset (integerPartitions n, isIPRefinement)
+
+
+-- Could also implement the Young lattice (or the part up to n)
+-- Of integer partitions <= n, ordered by inclusion
+-- Kassel, Turaev, Braid Groups, p202
+
+
+-- INTEGER COMPOSITIONS
+
+
 -- LATTICE OF SUBSPACES OF Fq^n
 
 subspaces fq n = [] : concatMap (flatsPG (n-1) fq) [0..n-1]
 -- note that flatsPG returns the subspaces as a matrix of row vectors in reduced row echelon form
 
--- inSpanRE m v returns whether the vector v is in the span of the matrix m, where m is required to be in row echelon form
+-- inSpanRE m v returns whether the vector v is in the span of the rows of the matrix m, where m is required to be in row echelon form
 isSubspace s1 s2 = all (inSpanRE s2) s1
 
 -- This is the projective geometry PG(n,q)
@@ -254,12 +295,6 @@ orderAuts1 poset = orderIsos poset poset
 -- What we really want is to return generators of the permutation group
 
 
-
-
-
-pairs (x:xs) = map (x,) xs ++ pairs xs -- TupleSections
-pairs [] = []
-
 -- |A linear extension of a poset is a linear ordering of the elements which extends the partial order.
 -- Equivalently, it is an ordering [x1..xn] of the underlying set, such that if xi <= xj then i <= j.
 isLinext (Poset (set,po)) set' = all (\(x,y) -> not (y `po` x)) (pairs set')
