@@ -1,7 +1,8 @@
 -- Copyright (c) 2006-2011, David Amos. All rights reserved.
 
 -- |A module for finding prime factors.
-module Math.NumberTheory.Factor (pfactors) where
+module Math.NumberTheory.Factor (module Math.NumberTheory.Prime,
+                                 pfactors) where
 
 import Math.NumberTheory.Prime
 import Data.Either (lefts)
