@@ -3,7 +3,8 @@
 {-# LANGUAGE NoMonomorphismRestriction #-}
 
 -- |A module providing functions to test for primality, and find next and previous primes.
-module Math.NumberTheory.Prime where
+module Math.NumberTheory.Prime (primes, isTrialDivisionPrime, isMillerRabinPrime,
+                                isPrime, notPrime, prevPrime, nextPrime) where
 
 import System.Random
 import System.IO.Unsafe
