@@ -4,7 +4,8 @@ module Math.Test.TAlgebras.TOctonions where
 
 import Test.QuickCheck
 
-import Math.Algebra.Field.Base
+import Math.Core.Field
+-- import Math.Algebra.Field.Base
 import Math.Algebras.VectorSpace
 import Math.Algebras.TensorProduct
 import Math.Algebras.Quaternions
