@@ -8,7 +8,8 @@ module Math.Test.TAlgebras.TTensorProduct where
 import Test.QuickCheck
 import Math.Algebras.VectorSpace
 import Math.Algebras.TensorProduct
-import Math.Algebra.Field.Base
+import Math.Core.Field
+-- import Math.Algebra.Field.Base
 import Math.Test.TAlgebras.TVectorSpace
 
 import Prelude as P
@@ -88,7 +89,7 @@ instance Category (Linear k) where
     id = Linear P.id
     (Linear f) . (Linear g) = Linear (f P.. g)
 
-instance Num k => Arrow (Linear k) where
+instance (Eq k, Num k) => Arrow (Linear k) where
     arr f = Linear (fmap f) -- requires nf call afterwards
     first (Linear f) = Linear f *** Linear P.id
     second (Linear f) = Linear P.id *** Linear f
