@@ -8,7 +8,8 @@ module Math.Test.TAlgebras.TVectorSpace where
 import Test.QuickCheck
 import Math.Algebras.VectorSpace
 import Math.Algebras.TensorProduct
-import Math.Algebra.Field.Base
+import Math.Core.Field
+-- import Math.Algebra.Field.Base
 
 -- import Control.Monad -- MonadPlus
 
@@ -40,10 +41,10 @@ instance Arbitrary Q where
                    d <- arbitrary :: Gen Integer
                    return (if d == 0 then fromInteger n else fromInteger n / fromInteger d)
 
-instance (Num k, Ord b, Arbitrary k, Arbitrary b) => Arbitrary (Vect k b) where
+instance (Eq k, Num k, Ord b, Arbitrary k, Arbitrary b) => Arbitrary (Vect k b) where
     arbitrary = do ts <- arbitrary :: Gen [(b, k)] -- ScopedTypeVariables
-                   return $ nf $ V $ take 10 ts
-                   -- we impose complexity bound of 10 terms, to avoid unbounded running time.
+                   return $ nf $ V $ take 3 ts
+-- we impose a complexity bound of 3 terms to limit to 27 terms when testing associativity and ensure reasonable running time
 
 prop_VecSpQn (a,b,x,y,z) = prop_VecSp (a,b,x,y,z)
     where types = (a,b,x,y,z) :: (Q, Q, Vect Q EBasis, Vect Q EBasis, Vect Q EBasis)
@@ -91,7 +92,7 @@ f .+. g = linear fg' where
 type LinFun k a b = [(a, Vect k b)]
 -- a way of representing a linear function as data
 
-linfun :: (Eq a, Ord b, Num k) => LinFun k a b -> Vect k a -> Vect k b
+linfun :: (Eq k, Num k, Eq a, Ord b) => LinFun k a b -> Vect k a -> Vect k b
 linfun avbs = linear f where
     f a = case lookup a avbs of
           Just vb -> vb
@@ -165,11 +166,11 @@ prop_BilinearQn1 f (a,u1,u2,v1,v2) = prop_Bilinear1 f (a,u1,u2,v1,v2)
     where types = (a,u1,u2,v1,v2) :: (Q, Vect Q EBasis, Vect Q EBasis, Vect Q EBasis, Vect Q EBasis)
 -}
 
-tensor :: (Num k, Ord a, Ord b) => Vect k (Either a b) -> Vect k (a, b)
+tensor :: (Eq k, Num k, Ord a, Ord b) => Vect k (Either a b) -> Vect k (a, b)
 tensor uv = nf $ V [( (a,b), x*y) | (a,x) <- u, (b,y) <- v]
     where V u = p1 uv; V v = p2 uv
 
-bilinear :: (Num k, Ord a, Ord b, Ord c) =>
+bilinear :: (Eq k, Num k, Ord a, Ord b, Ord c) =>
     ((a, b) -> Vect k c) -> Vect k (Either a b) -> Vect k c
 bilinear f = linear f . tensor
 
@@ -177,7 +178,7 @@ dot = bilinear (\(a,b) -> if a == b then return () else zero)
 
 polymult = bilinear (\(E i, E j) -> return (E (i+j)))
 
-prop_Bilinear :: (Num k, Ord a, Ord b, Ord t) =>
+prop_Bilinear :: (Eq k, Num k, Ord a, Ord b, Ord t) =>
      (Vect k (Either a b) -> Vect k t) -> (k, Vect k a, Vect k a, Vect k b, Vect k b) -> Bool
 prop_Bilinear f (a,u1,u2,v1,v2) =
     prop_Linear (\v -> f (u1 `dsume` v)) (a,v1,v2) &&
